function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

import { objectReduce } from 'fast-loops';

var defaultGenerator = function defaultGenerator(identifierName, index) {
  return index.toString();
};

var isRenderer = function isRenderer(renderer) {
  return renderer && renderer.renderRule;
};

export default function identifier() {
  var config = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var _config$prefix = config.prefix,
      prefix = _config$prefix === void 0 ? 'fela-identifier' : _config$prefix;
  var _config$generator = config.generator,
      generator = _config$generator === void 0 ? defaultGenerator : _config$generator;
  var idRepository = {};
  var index = 0;
  var enhanced = false;
  return function (identifierName) {
    if (isRenderer(identifierName)) {
      enhanced = true;
      var renderer = identifierName;
      var existingFilterClassName = renderer.filterClassName.bind(renderer);
      var existingRenderRule = renderer.renderRule.bind(renderer);

      renderer.filterClassName = function (className) {
        return existingFilterClassName(className) && !idRepository[className];
      };

      renderer.renderRule = function (rule, props) {
        var idClassNames = [];
        var style = rule(props, renderer);
        var cleanedStyle = objectReduce(style, function (newStyle, value, key) {
          var identifierKey = key.replace(/^--/, '');

          if (idRepository[identifierKey]) {
            idClassNames.push(idRepository[identifierKey].className);
            return newStyle;
          }

          return _objectSpread(_objectSpread({}, newStyle), {}, _defineProperty({}, key, value));
        }, {});

        var modifiedRule = function modifiedRule() {
          return cleanedStyle;
        };

        Object.assign(modifiedRule, rule);
        return [].concat(idClassNames, [existingRenderRule(modifiedRule)]).join(' ');
      };

      return renderer;
    }

    if (!enhanced) {
      throw new Error('You are trying to create a new identifier before you used the identifier as an enhancer. ' + 'Please add the identifier as an enhancer in the configuration of the renderer.');
    }

    var identifierKey = [prefix, identifierName, generator(identifierName, index++)].filter(function (chunk) {
      return chunk;
    }).join('-');

    var identifierRule = function identifierRule() {
      return _defineProperty({}, "--".concat(identifierKey), '');
    };

    identifierRule.className = identifierKey;

    identifierRule.toString = function () {
      return identifierKey;
    };

    idRepository[identifierKey] = identifierRule;
    return identifierRule;
  };
}